/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.custom;

import org.wso2.registry.Registry;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.users.UserRealm;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

public abstract class CustomUIProcessor {

    private static final String VIEW_SEPARATOR = ":";

    protected UserRegistry registry;

    protected UserRealm userRealm;

    public abstract String getDefaultCreateView();

    public abstract String getDefaultResourceView();

    public abstract Map <String, String> getCreateViews();

    public abstract Map <String, String> getResourceViews();

    /**
     * Processes the requests originated from custom UIs. Content sending method, parameters in the
     * request, etc are entirely decided by the implementation.
     *
     * @param request
     * @return
     */
    public abstract CustomUIBean process(String command, HttpServletRequest request)
            throws RegistryException;

    public abstract CustomUIBean processCreateView(HttpServletRequest request, String viewKey)
            throws RegistryException;

    public abstract CustomUIBean processResourceView(
            String path, String viewKey, HttpServletRequest request) throws RegistryException;

    public void setRequest(HttpServletRequest request) throws RegistryException {

        registry = CommonUtil.getUserRegistry(request);
        userRealm = registry.getUserRealm();
    }
}
